Minifocus.js
============

`Minifocus` is a minimal focus-shifting plugin for `jQuery`. It helps handle focus-shifting logics among a
group of elements when a key is pressed.

`Minifocus` relies on [Map][1] and [KeyboardEvent.key][2] to work. If you want to use `Minifocus` in browsers
that may not support these, you may need some polyfill libraries.


Getting started
---------------

See `examples/index.html` for examples.


How it works
------------

`Minifocus` uses `data-mfgroup` attribute to declare a region in which it will manage focus-shifting logics,
and uses `data-mfindex` attribute to define the focus-shifting order among the child elements.

Assume `n` elements

```
<element data-mfindex="1">
<element data-mfindex="2">
...
<element data-mfindex="n">
```

are within the same parent `<element data-mfgroup>`, then the focus-shifting order used by `Minifocus` is

```
element 1, element 2, ..., element n, element 1, ...
```

By default, you can use `Tab` and `Enter` key to shift focus. If an element has default action when `Enter`
is pressed (e.g. starting a new line in a `textarea`), you can only use `Tab` key to shift focus when it
has focus. You can also customize the focus-shifting rules using the `$.minifocus.setRuleForSelector` API
(see below).

Note that the value of `data-mfindex` must be continuous, and start from `1`.

You can have multiple `data-mfgroup` within one document. Every focus-shifting loop is located in one
`data-mfgroup`.

`Minifocus` prevents the default `Tab` behavior by default, which means `tabindex` attribute will not work,
and you cannot shift focus from elements without `data-mfindex` by `Tab` key. You can enable or disable this
using the `$.minifocus.setPreventDefaultTabBehavior` API (see below).


API
---

- `$.minifocus.init()`

Let `Minifocus` manage focus-shifting logics within all `data-mfgroup`s. Called automatically on `document`'s
`ready` event.

- `$.minifocus.setRuleForSelector(selector, method)`

Setup customized rule for `selector`. When `KeyboardEvent` happens on a focused element with `data-mfindex`,
and the element matches `selector`, then `Minifocus` will call `method` to test if it should shift focus to
another element. Focus is shifted only if `method` returns `true`. The `method` accepts two arguments `key`
and `$focusSrc`, `key` is the string representation of the pressed key (see [the full list of key values][3]);
`$focusSrc` is the element that receives the `KeyboardEvent`, wrapped in a `jQuery` object.

- `$.minifocus.removeRuleForSelector(selector)`

Remove the customized rule for `selector`.

- `$.minifocus.setPreventDefaultTabBehavior(preventDefaultBehavoir)`

Set if you want `Minifocus` to prevent the default behavior when `Tab` is pressed.


Build
------------

`npm install`

`npm run build`


License
-------

MIT


[1]: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map
[2]: https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key
[3]: https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values